/*
Although FLTK is a cross-platform C++ GUI toolkit, rannu includes OS-specific functions:
1. Creating the rannu icon using CreateIcon().
2. Starting PDF software.
3. Measuring screen size.
4. Using wildcards ( *, ? ) to find files.
5. Showing where the cursor is.

*/

#include "OS-specific.h"

namespace {
    HICON hIcon;
}

extern HINSTANCE fl_display;
extern int TITLE_BAR_HEIGHT;
extern ManagePane *managePane;
extern Rannu *rannu;
extern VersionPane *versionPane;

bool createRemoveErrorText() {
    using namespace std;

    bool found;
    const int FILE_TYPE_COUNT = 8;
    const string e0("err*.txt");
    const string e1("pic*.txt");
    const string e2("tru*.bin");
    const string e3("und*.bin");
    const string e4("");
    const string e5("");
    const string e6("");
    const string e7("");
    string file_type[FILE_TYPE_COUNT];
    HANDLE find_first_next_handle;
    ofstream remove_e(REMOVE_ERROR_TEXT);
    if( remove_e.fail() ) {
        ofstream file(ERROR_TEXT);
        file << "ofstream remove_e(REMOVE_ERROR_TEXT)" << endl;
        return false;
    }

    WIN32_FIND_DATA find_data;

    file_type[0] = e0;
    file_type[1] = e1;
    file_type[2] = e2;
    file_type[3] = e3;
    file_type[4] = e4;
    file_type[5] = e5;
    file_type[6] = e6;
    file_type[7] = e7;
    for(int i = 0; i < FILE_TYPE_COUNT; ++i) {
        if( file_type[i].length() == 0 ) {
            break;
        }

        find_first_next_handle = ::FindFirstFile( file_type[i].c_str(), &find_data);
        if(find_first_next_handle == INVALID_HANDLE_VALUE) {
            continue;
        }

        found = true;
        while(found) {
            remove_e << find_data.cFileName << endl;
            found = ::FindNextFile(find_first_next_handle, &find_data);
        }

        ::FindClose(find_first_next_handle);
    }

    return true;
}

bool Rannu::where() {
    using namespace std;

    const int ADJUSTER = 50;
    const int BORDER = 1;
    const int BOTTOM_ADJUSTER = 82;
    const int WALL = 2;
    const string INSIDE("inside");
    const string OUTSIDE("outside");
    const string PARTITION(" | ");
    POINT pos;

    ::GetCursorPos(&pos);

    bool inside = false;
    int upper_left_x = x() + BORDER;
    int upper_left_y = y() + ADJUSTER;
    int upper_right_x = upper_left_x + w() - WALL;
    int lower_left_y = upper_left_y + h() - BOTTOM_ADJUSTER;
    string here = OUTSIDE;
    if(upper_left_x < pos.x && pos.x < upper_right_x) {
        if(upper_left_y < pos.y && pos.y < lower_left_y) {
            here = INSIDE;
            inside = true;
        }

    }

    int q = statusBarText.find(PARTITION);
    if(q != string::npos) {
        statusBarText = statusBarText.substr(0, q);
    }

    stringstream os;
    os << statusBarText << PARTITION << pos.x << " : " << pos.y << " ( " << here << " ) ";
    string s = os.str();
    statusBarMessage(s);
    return inside;
}

int executeProgram(char *p, char *q) {
    char FileName[FILE_NAME_SIZE];
    char Parameters[PARAMETER_SIZE];
    strcpy(FileName, p);
    strcpy(Parameters, q);

    SHELLEXECUTEINFO execinfo;
    memset(&execinfo, 0, sizeof(execinfo) );
    execinfo.cbSize = sizeof(execinfo);
    execinfo.fMask = SEE_MASK_NOCLOSEPROCESS;
    execinfo.lpFile = FileName;
    execinfo.lpParameters = Parameters;
    execinfo.lpVerb = "open";
    execinfo.nShow = SW_SHOWDEFAULT;
    return ::ShellExecuteEx(&execinfo);
}

int ManagePane::centering() {
    const int HALF_WIDTH = MANAGE_WIDTH/2;
    const int HALF_HEIGHT = ( MANAGE_HEIGHT + TITLE_BAR_HEIGHT )/2;
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += TITLE_BAR_HEIGHT;

    const int CENTER_X = SCREEN_WIDTH/2 - HALF_WIDTH;
    int center_y = screen_height/2 - HALF_HEIGHT;
    TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);
    center_y += TITLE_BAR_HEIGHT;

    managePane->resize(CENTER_X, center_y, MANAGE_WIDTH, MANAGE_HEIGHT);

    int BORDER_X = ::GetSystemMetrics(SM_CXFIXEDFRAME);
    int BORDER_Y = ::GetSystemMetrics(SM_CYFIXEDFRAME);
    int border = BORDER_X > BORDER_Y ? BORDER_X : BORDER_Y;
    return border;
}

int Rannu::centering() {
    const int HALF_WIDTH = rannu_w/2;
    const int HALF_HEIGHT = ( rannu_h + TITLE_BAR_HEIGHT )/2;
    int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += TITLE_BAR_HEIGHT;

    const int CENTER_X = SCREEN_WIDTH/2 - HALF_WIDTH;
    int center_y = screen_height/2 - HALF_HEIGHT;
    TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);
    center_y += TITLE_BAR_HEIGHT;

    rannu->resize(CENTER_X, center_y, rannu_w, rannu_h);
    return SCREEN_WIDTH;
}

std::string currentDirectory() {
    const DWORD DIRECTORY_BUFFER_SIZE = 1024;
    char buffer[DIRECTORY_BUFFER_SIZE];
    ::GetCurrentDirectory(DIRECTORY_BUFFER_SIZE, (LPTSTR)buffer );

    std::string current_directory(buffer);
    return current_directory;
}

void create_rannu_icon(Fl_Window *window) {
    const int ICON_HEIGHT = GetSystemMetrics(SM_CYICON); // 32
    const int ICON_WIDTH = GetSystemMetrics(SM_CXICON); // 32
    const int EVEN_BYTE =  (ICON_WIDTH + 7)/8 * 8; // 32
    const int BYTE_COUNT = EVEN_BYTE/8; // 4
    HINSTANCE hInstance = fl_display;
    int i, j, k;
    int nIconBytes = (EVEN_BYTE/8) * ICON_HEIGHT; // 128
    LPBYTE a, x;
    LPBYTE maskAND, maskXOR;

    unsigned char ub[128] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
    unsigned char uc[128] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x3F, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x3F, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

    maskAND = (LPBYTE)GlobalAlloc(GPTR, nIconBytes);
    maskXOR = (LPBYTE)GlobalAlloc(GPTR, nIconBytes);

    a = maskAND;
    k = 0;
    x = maskXOR;
    for(i = 0; i < BYTE_COUNT; i++) {
        for(j = 0; j < ICON_HEIGHT; j++) {
            *a++ = ub[k];
            *x++ = uc[k];
            k++;
        }

    }

    hIcon = CreateIcon(hInstance, ICON_WIDTH, ICON_HEIGHT, 1, 1, maskAND, maskXOR);
    GlobalFree(maskAND);
    GlobalFree(maskXOR);

    window->icon(hIcon);
}

void destroy_rannu_icon() {
    DestroyIcon(hIcon);
}

void VersionPane::centering() {
    const int HALF_WIDTH = VERSION_WIDTH/2;
    const int HALF_HEIGHT = ( VERSION_HEIGHT + TITLE_BAR_HEIGHT )/2;
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += TITLE_BAR_HEIGHT;

    const int CENTER_X = SCREEN_WIDTH/2 - HALF_WIDTH;
    int center_y = screen_height/2 - HALF_HEIGHT;
    TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);
    center_y += TITLE_BAR_HEIGHT;

    versionPane->resize(CENTER_X, center_y, VERSION_WIDTH, VERSION_HEIGHT);
}
