/*
Rannu FLTK Version 1.01
Producing random-number tables
Copyright 2017 Sogaya

GCC
FLTK 1.3.0

*/

#include "rannuMain.h"

char buffer[ MAIN_BUFFER_SIZE + 1 ];
int TITLE_BAR_HEIGHT = 19;
long untouched;
MANAGE_MEMO manage_memo; // rannu.h
MANAGE_MEMO *mm = &manage_memo;
ManagePane *managePane;
Rannu *rannu;
VersionPane *versionPane;

namespace {
    using namespace std;          
          
    const bool ARITHMETIC_MEAN = true;
    const bool VARIANCE = false;
    const string CLOSE_BRACKET(" )");
    const string RANNU_TEXT("rannu.txt");
    const string ZERO("0");
    const int MERSENNE_TWISTER = 1;
    const int XORSHIFT = 6;

    bool csv, grouping, produce_rannu_text, showWhere;
    string error, FULL_PATH_NAME, pathName;
    int from, interval_0, interval_1, lineCount, rannu_h, rannu_w, rannu_x, rannu_y, prng, whichCorner;
    long kilo, seed;

    const string m0(          ""); // m[0], FULL_PATH_NAME includes MANAGE_TEXT.
    const string m1(         "0"); // m[1], csv: RANNU_CSV
    const string m2(      "C:\\"); // m[2], pathName includes the directory name (C:\) and RANNU_TEXT.
    const string m3(        "32"); // m[3], kilo: RANNU_TEXT size (kilobytes, KB)
    const string m4(         "1"); // m[4], interval_0: Loto-7
    const string m5(        "37"); // m[5], interval_1: Loto-7
    const string m6(         "0"); // m[6], grouping: interval of 0-63
    const string m7(       "400"); // m[7], from: idling
    const string m8(    "123456"); // m[8], seed: pseudorandom number generator (PRNG)
    const string m9("2147483647"); // m[9]
}

bool Rannu::header(std::string & pathName) {
    using namespace std;

    ifstream in( pathName.c_str() );
    if( in.fail() ) {
        error = "e15: ifstream in( " + pathName + CLOSE_BRACKET;
        return false;
    }

    stringstream os;
    char buffer[BUFFER_SIZE];
    for(int i = 0; i < FIRST_PART_OF_FILE; ++i) {
        if( in.getline(buffer, BUFFER_SIZE) ) {
            os << buffer << NEW_LINE;
        } else {
            break;
        }

    }

    string s = os.str();
    areaTextBuffer->text( s.c_str() );

    string PATH_NAME( pathName.c_str() );
    statusBarMessage(PATH_NAME);
    return true;
}

bool Rannu::readManageText() {
    using namespace std;

    char c[BUFFER_SIZE];
    string *p = new string("");
    ifstream in( FULL_PATH_NAME.c_str() );
    if( in.fail() ) {
        error = "e11: ifstream in( " + FULL_PATH_NAME + CLOSE_BRACKET;
        createErrorText(error);
        return false;
    }

    m[0] = FULL_PATH_NAME;
    for(int line = 1; line <= MANAGE_TEXT_LINE_COUNT; line += 1) {
        if( in.getline(c, BUFFER_SIZE) ) {
            delete p;
            p = new string(c);
            m[line] = *p;
        } else {
            break;
        }

    }

    in.close();
    stringArrayToValues();
    delete p;
    return true;
}

bool Rannu::setting(VersionPane *v) {
    const int COMBO_HEIGHT = STATUS_BAR_HEIGHT;
    const int COMBO_WIDTH = MAIN_WIDTH/2;
    const int TEXT_AREA_WIDTH = MAIN_WIDTH;
    const int TEXT_AREA_Y = MENU_BAR_HEIGHT + COMBO_HEIGHT;
    const int TEXT_AREA_HEIGHT = MAIN_HEIGHT - STATUS_BAR_HEIGHT - TEXT_AREA_Y;
    const std::string C_RANNU_FLTK_MANAGE_TEXT("C:\\rannuFLTK\\manage.txt");
    const std::string MT("Mersenne Twister (MT)");
    const std::string X("xorshift");

    versionPane = v;

// Alt: A B C D E F G H I L M N O R S T U V W X Y 
// Ctrl: I K N P Q Y 
    Fl_Menu_Item menuitems[] = {
    { "&File", 0, 0, 0, FL_SUBMENU },
    { "Count      ", 0, (Fl_Callback *)count_cb, 0 },
    { "&CSV      ", 0, (Fl_Callback *)csv_cb, 0 },
    { "&Exit      ", FL_COMMAND + 'q', (Fl_Callback *)quit_cb, 0 },
    { "M&anage      ", FL_COMMAND + 'i', (Fl_Callback *)manage_cb, 0 },
    { "Mean      ", FL_COMMAND + 'n', (Fl_Callback *)mean_cb, 0 },
    { "&Text      ", FL_COMMAND + 'k', (Fl_Callback *)text_cb, 0 },
    { 0 },
    { "&Setting", 0, 0, 0, FL_SUBMENU },
    { "&MT        ", 0, (Fl_Callback *)MT_cb, 0 },
    { "&xorshift        ", 0, (Fl_Callback *)xorshift_cb, 0, FL_MENU_DIVIDER  },
    { "Cop&y        ", 0, (Fl_Callback *)copy_cb, 0, FL_MENU_DIVIDER  },
    { "&Grouping        ", 0, (Fl_Callback *)grouping_cb, 0 },
    { "&Bingo-5        ", 0, (Fl_Callback *)bingo5_cb, 0 },
    { "L&oto-6        ", 0, (Fl_Callback *)loto6_cb, 0 },
    { "Loto-7        ", 0, (Fl_Callback *)loto7_cb, 0 },
    { "Miniloto        ", 0, (Fl_Callback *)miniloto_cb, 0 },
    { 0 },
    { "&Window", 0, 0, 0, FL_SUBMENU },
    { "Center", 0, (Fl_Callback *)center_cb, 0 },
    { "Cor&ner", 0, (Fl_Callback *)corner_cb, 0 },
    { "D&iagonal", 0, (Fl_Callback *)diagonal_cb, 0 },
    { "&Large    ", FL_COMMAND + 'p', (Fl_Callback *)large_cb, 0 },
    { "&Right", 0, (Fl_Callback *)right_cb, 0 },
    { "Small", 0, (Fl_Callback *)small_cb, 0 },
    { "Stan&dard    ", FL_COMMAND + 'y', (Fl_Callback *)standard_cb, 0 },
    { "&Under", 0, (Fl_Callback *)under_cb, 0 },
    { 0 },
    { "&Help", 0, 0, 0, FL_SUBMENU },
    { "&Version  ", 0, (Fl_Callback *)version_cb, 0 },
    { 0 },
    { 0 }
    };

    menu = new Fl_Menu_Bar(0, 0, MAIN_WIDTH, MENU_BAR_HEIGHT);
    menu->box(FL_FLAT_BOX);
    menu->copy(menuitems);

    int y = MAIN_HEIGHT - STATUS_BAR_HEIGHT;
    statusBarTextBuffer = new Fl_Text_Buffer();
    statusBar = new Fl_Text_Display(0, y, MAIN_WIDTH, STATUS_BAR_HEIGHT, "");
    statusBar->buffer(statusBarTextBuffer);
    statusBar->textsize(20);
    statusBar->box(FL_FLAT_BOX);
    statusBarText = C_RANNU_FLTK_MANAGE_TEXT;
    statusBarTextBuffer->text( C_RANNU_FLTK_MANAGE_TEXT.c_str() );

    comboTextBuffer = new Fl_Text_Buffer();
    combo = new Fl_Text_Display(0, MENU_BAR_HEIGHT, COMBO_WIDTH, COMBO_HEIGHT, "");
    combo->buffer(comboTextBuffer);
    combo->textsize(20);
    combo->box(FL_FLAT_BOX);

    prng = mm->prng;
    if(prng == XORSHIFT) {
        comboTextBuffer->text( X.c_str() );
    } else {
        comboTextBuffer->text( MT.c_str() );
    }

    countButton = new Fl_Button(COMBO_WIDTH, MENU_BAR_HEIGHT, BUTTON_WIDTH, BUTTON_HEIGHT, "Count");
    countButton->callback( (Fl_Callback *)count_cb, this);

    int x = MAIN_WIDTH - BUTTON_WIDTH;
    largeButton = new Fl_Button(x, MENU_BAR_HEIGHT, BUTTON_WIDTH, BUTTON_HEIGHT, "Large");
    largeButton->callback( (Fl_Callback *)large_cb, this);

    areaTextBuffer = new Fl_Text_Buffer();
    textArea = new Fl_Text_Display(0, TEXT_AREA_Y, TEXT_AREA_WIDTH, TEXT_AREA_HEIGHT, "");
    textArea->buffer(areaTextBuffer);
    textArea->textsize(16);
    textArea->box(FL_DOWN_BOX);
    areaTextBuffer->text("");
    return true;
}

bool Rannu::textHandler() {
    bool result = false;

    checkManageMemory();

// Producing a random number table
    RandomNumber *rn = new RandomNumber(kilo, seed, from, interval_0, interval_1, lineCount, prng, csv, grouping, produce_rannu_text);
    if( rn->createRannuText(pathName) ) {
        result = true;
    } else {
        error = "e13: rn->createRannuText( " +  pathName + CLOSE_BRACKET;
        createErrorText(error);
    }

    delete rn;
    header(pathName);
    return result;
}

int main (int argc, char *argv[]) {
    using namespace std;

    char *stop;
    const bool BUILDING = true;
    const bool WITH_COMMENTS = true;
    const bool WITHOUT_COMMENTS = false;
    const string BINGO_5 = "-b";
    const string EMPTY = "";
    const string FROM_TERM = "-f"; // idling
    const string GROUPING = "-g";
    const string INTERVAL = "-i";
    const string KILOBYTE_TERM = "-k";
    const string LINES = "-L";
    const string LOTO_6 = "-6";
    const string LOTO_7 = "-7";
    const string MINILOTO = "-m";
    const string PRNG_MT = "-pm";
    const string PRNG_XORSHIFT = "-px";
    const string REGARDLESS_OF_LINES = "-L-";
    const string SEED_TERM = "-s";
    const string version_comments = "-VC";
    const string version_guide = "-V";
    const string WHERE("-w");
    const int ERROR_CODE = 1;

    int d;
    Numbered *n = new Numbered("");
    string *p = new string("");
    string argument;
    string manage_text_name(MANAGE_TEXT);
    string m[ MANAGE_TEXT_LINE_COUNT + 1 ];
    m[0] = m0; m[1] = m1; m[2] = m2; m[3] = m3; m[4] = m4; m[5] = m5; m[6] = m6; m[7] = m7; m[8] = m8; m[9] = m9;
    prng = MERSENNE_TWISTER; // MT
    produce_rannu_text = true;

// C:\rannuFLTK\manage.txt for example
    string firstPart = currentDirectory(); // OS-specific.h and OS-specific.cpp
    NAMES a = driveDirectoryBase(firstPart);
    string SEPARATOR = a.separator;
    string root = a.name[1];
    if (firstPart == root) { // C:\ for example
        FULL_PATH_NAME = root + manage_text_name;
    } else { // C:\rannuFLTK for example
        FULL_PATH_NAME = firstPart + SEPARATOR + manage_text_name;
    }

    ifstream file( FULL_PATH_NAME.c_str() );
    if( file.fail() ) { // MANAGE_TEXT
// Creating MANAGE_TEXT
        ofstream manage_text( FULL_PATH_NAME.c_str() );
        for(int line = 1; line <= MANAGE_TEXT_LINE_COUNT; line += 1 ) {
            manage_text << m[line] << endl;
        }

        return ERROR_CODE;
    } else {
        file.close();
    }

// Reading MANAGE_TEXT
    char c[BUFFER_SIZE];
    ifstream in( FULL_PATH_NAME.c_str() );
    m[0] = FULL_PATH_NAME;
    for(int line = 1; line <= MANAGE_TEXT_LINE_COUNT; line += 1) {
        if( in.getline(c, BUFFER_SIZE) ) {
            delete p;
            p = new string(c);
            m[line] = *p;
        } else {
            break;
        }

    }

    in.close();
    csv = (ZERO == m[1]) ? false : true;
    pathName = m[2] + RANNU_TEXT;
    kilo = strtol( m[3].c_str(), &stop, 10);
    const long based_on_m3 = kilo;

    interval_0 = stringToInteger(m[4]);
    const int based_on_m4 = interval_0;

    interval_1 = stringToInteger(m[5]);
    const int based_on_m5 = interval_1;

    grouping = (ZERO == m[6]) ? false : true;
    from = stringToInteger(m[7]);
    const int based_on_m7 = from;

    seed = strtol( m[8].c_str(), &stop, 10);
    const long based_on_m8 = seed;

    lineCount = stringToInteger(m[9]);
    const int based_on_m9 = lineCount;

    rannu_h = MAIN_HEIGHT;
    rannu_w = MAIN_WIDTH;
    rannu_x = MAIN_X;
    rannu_y = MAIN_Y;

    for(int i = 0; i < argc; ++i) {
        delete p;
        p = new string(argv[i]);
        argument = *p;

        if(argument == BINGO_5) {
            interval_1 = 40;
        } else if(argument == GROUPING) {
            grouping = true;
        } else if(argument == LOTO_6) {
            interval_1 = 43;
        } else if(argument == LOTO_7) {
            interval_1 = 37;
        } else if(argument == MINILOTO) {
            interval_1 = 31;
        } else if(argument == PRNG_MT) {
            prng = MERSENNE_TWISTER;
        } else if(argument == PRNG_XORSHIFT) {
            prng = XORSHIFT;
        } else if(argument == REGARDLESS_OF_LINES) {
            lineCount = LARGE_NUMBER;
        } else if(argument == version_guide) {
            guide(WITHOUT_COMMENTS);
            delete n; delete p;
            return 1;
        } else if(argument == version_comments) {
            guide(WITH_COMMENTS);
            delete n; delete p;
            return 1;
        } else if(argument == WHERE) {
            showWhere = true;
        } else {
            d = argument.find(FROM_TERM);
			if(d != string::npos) {
                delete n;
                n = new Numbered(argv[i]);
                from = n->number();
                if( n->parameter() == FROM_TERM ) {
                    continue;
                }

                from = based_on_m7;
            }

            d = argument.find(INTERVAL);
			if(d != string::npos) {
                delete n;
                n = new Numbered(argv[i]);
                interval_1 = n->number();
                if( n->trimmed_sequence() ) {
                    interval_0 = n->subnumber();
                    if( n->parameter() == INTERVAL ) {
                        continue;
                    }

                }

                interval_0 = based_on_m4;
                interval_1 = based_on_m5;
            }

            d = argument.find(KILOBYTE_TERM);
			if(d != string::npos) {
                delete n;
                n = new Numbered(argv[i]);
                kilo = (long) n->number();
                if( n->parameter() == KILOBYTE_TERM ) {
                    continue;
                }

                kilo = based_on_m3;
            }

            d = argument.find(LINES);
			if(d != string::npos) {
                delete n;
                n = new Numbered(argv[i]);
                lineCount = n->number();
                if( n->parameter() == LINES ) {
                    continue;
                }

                lineCount = based_on_m9;
            }

            d = argument.find(SEED_TERM);
			if(d != string::npos) {
                delete n;
                n = new Numbered(argv[i]);
                seed = (long) n->number();
                if( n->parameter() == SEED_TERM ) {
                    continue;
                }

                seed = based_on_m8;
            }

        }

    }

    if(grouping) {
        interval_0 = 0;
        interval_1 = 63;
    }

    mm->csv = csv;
    mm->from = from;
    mm->FULL_PATH_NAME = FULL_PATH_NAME;
    mm->grouping = grouping;
    mm->interval_0 = interval_0;
    mm->interval_1 = interval_1;
    mm->kilo = kilo;
    mm->lineCount = lineCount;
    mm->rannu_h = rannu_h;
    mm->rannu_w = rannu_w;
    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->pathName = pathName;
    mm->pleaseCheck = true;
    mm->prng = prng;
    mm->produce_rannu_text = produce_rannu_text;
    mm->seed = seed;

    versionPane = new VersionPane(VERSION_WIDTH, VERSION_HEIGHT, "Version");
    versionPane->setting();
    versionPane->end();
    versionPane->centering();
    versionPane->resizable(NULL);
    versionPane->set_modal();
    versionPane->hide();

    managePane = new ManagePane(MANAGE_WIDTH, MANAGE_HEIGHT, "Manage");
    managePane->setting(BUILDING);
    managePane->end();
    managePane->centering();
    managePane->resizable(NULL);
    managePane->set_modal();
    managePane->hide();

    rannu = new Rannu(MAIN_WIDTH, MAIN_HEIGHT, "Rannu");
    rannu->setting(versionPane);
    rannu->end();
    rannu->resize(MAIN_X, MAIN_Y, MAIN_WIDTH, MAIN_HEIGHT);
    rannu->resizable(NULL);
    rannu->standardHandler();

    create_rannu_icon(rannu);
    rannu->show();

    untouched = modified_file(MANAGE_TEXT);
    Fl::add_timeout(NONSAMPLING_PERIOD, timer_cb);
    return Fl::run();
}

long modified_file(const char *s) {
    long g;
    struct stat file_time;
    struct tm *p;

    if(! stat(s, &file_time) ) {
        p = gmtime( &(file_time.st_mtime) );
        g = p->tm_sec + 60*p->tm_min + 60*60*p->tm_hour + 60*60*24*p->tm_yday;
    } else {
        g = 0L;
    }

    return g;
}

Rannu::Rannu(int W, int H, const char *t) : Fl_Window(W, H, t), COMMA(","), EMPTY(""), NEW_LINE("\n"), STATUS_BAR_TEXT_LIMIT(80), TEXT_LIMIT(3) {
    const int MERSENNE_TWISTER = 1;
    m[0] = m0; m[1] = m1; m[2] = m2; m[3] = m3; m[4] = m4; m[5] = m5; m[6] = m6; m[7] = m7; m[8] = m8; m[9] = m9;
    prng = MERSENNE_TWISTER;
    produce_rannu_text = true;
    rannu_h = H;
    rannu_w = W;
} 

void bingo5_cb(Fl_Widget *, void *) {
    rannu->lotoHandler(40);
}

void center_cb(Fl_Widget *, void *) {
    rannu->centerHandler();
}

void copy_cb(Fl_Widget *, void *) {
    rannu->copyHandler();
}

void corner_cb(Fl_Widget *, void *) {
    rannu->cornerHandler();
}

void count_cb(Fl_Widget *, void *) {
    rannu->cou99(VARIANCE);
}

void csv_cb(Fl_Widget *, void *) {
    mm->csv = true;
    mm->pleaseCheck = true;
    rannu->textHandler();
}

void diagonal_cb(Fl_Widget *, void *) {
    rannu->diagonalHandler();
}

void grouping_cb(Fl_Widget *, void *) {
    rannu->groupingHandler();
}

void guide(bool comments) {
    using namespace std;

    if(comments) {
        ofstream file(VERSION_TEXT);
        file << "rannu FLTK Version " << VERSION << " \n" << endl 
            << "rannu        # Show the Rannu window. " << endl 
            << "rannu -6     # Loto-6 " << endl 
            << "rannu -7     # Loto-7 " << endl 
            << "rannu -b     # Bingo-5 " << endl 
            << "rannu -f400  # idling " << endl 
            << "rannu -g     # interval of 0 to 63 " << endl 
            << "rannu -i1-37 # interval of 1 to 37 " << endl 
            << "rannu -k32   # 32 kilobytes " << endl 
            << "rannu -L20   # 20 lines " << endl 
            << "rannu -L-    # Produce rannu.txt regardless of lines. " << endl 
            << "rannu -m     # Miniloto " << endl 
            << "rannu -pm    # MT " << endl 
            << "rannu -px    # xorshift " << endl 
            << "rannu -s123456 # seed " << endl 
            << "rannu -s0    # Time is used as seed. " << endl 
            << "rannu -V     # version " << endl 
            << "rannu -VC    # version with comments " << endl 
            << "rannu -w     # Show where the cursor is. " << endl;
    } else {
        ofstream file(VERSION_TEXT);
        file << "rannu FLTK Version " << VERSION << " \n" << endl 
            << "rannu        " << endl 
            << "rannu -6     " << endl 
            << "rannu -7     " << endl 
            << "rannu -b     " << endl 
            << "rannu -f400  " << endl 
            << "rannu -g     " << endl 
            << "rannu -i1-37 " << endl 
            << "rannu -k32   " << endl 
            << "rannu -L20   " << endl 
            << "rannu -L-    " << endl 
            << "rannu -m     " << endl 
            << "rannu -pm    " << endl 
            << "rannu -px    " << endl 
            << "rannu -s123456 " << endl 
            << "rannu -s0    " << endl 
            << "rannu -V     " << endl 
            << "rannu -VC    " << endl 
            << "rannu -w     " << endl;
    }

}

void help_cb(Fl_Widget *, void *) {
    versionPane->helpHandler();
}

void large_cb(Fl_Widget *, void *) {
    rannu->largeHandler();
}

void loto6_cb(Fl_Widget *, void *) {
    rannu->lotoHandler(43);
}

void loto7_cb(Fl_Widget *, void *) {
    rannu->lotoHandler(37);
}

void manage_cb(Fl_Widget *, void *) {
    rannu->manageHandler();
}

void mean_cb(Fl_Widget *, void *) {
    rannu->cou99(ARITHMETIC_MEAN);
}

void miniloto_cb(Fl_Widget *, void *) {
    rannu->lotoHandler(31);
}

void MT_cb(Fl_Widget *, void *) {
    const int MERSENNE_TWISTER = 1;
    rannu->selectingPRNG(MERSENNE_TWISTER);
}

void quit_cb(Fl_Widget *, void *) {
    destroy_rannu_icon();
    exit(0);
}

void Rannu::centerHandler() {
    checkManageMemory();
    whichCorner = 0;

    screen_width = centering();
    standard_main_x = x();
    standard_main_y = y();
    rannu_x = standard_main_x;
    rannu_y = standard_main_y;

    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->whichCorner = whichCorner;
}

void Rannu::checkManageMemory() {
    if(mm->pleaseCheck) {
        csv = mm->csv;
        from = mm->from;
        grouping = mm->grouping;
        interval_0 = mm->interval_0;
        interval_1 = mm->interval_1;
        kilo = mm->kilo;
        lineCount = mm->lineCount;
        rannu_h = mm->rannu_h;
        rannu_w = mm->rannu_w;
        rannu_x = mm->rannu_x;
        rannu_y = mm->rannu_y;
        pathName = mm->pathName;
        seed = mm->seed;

        mm->pleaseCheck = false;
    }

}

void Rannu::copyHandler() {
    const int CLIPBOARD = 1;
    int length = statusBarText.length();
    Fl::copy( statusBarText.c_str(), length, CLIPBOARD);
}

void Rannu::cornerHandler() {
    checkManageMemory();
    whichCorner = 1;
    rannu_x = CORNER_X;
    rannu_y = CORNER_Y;
    resize(rannu_x, rannu_y, rannu_w, rannu_h);

    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->whichCorner = whichCorner;
}

void Rannu::cou99(bool arithmetic) {
    using namespace std;

    bool analysis = !arithmetic;
    char file_name[FILE_NAME_SIZE];
    const string COLON_GAP(": ");
    Count99 *c = new Count99();
    double d = (double)(interval_0 + interval_1);
    double m = d/2.0;
    int columnCount, n;
    string floating_point_number, s;

    pathName = mm->pathName;
    strcpy(file_name, pathName.c_str() );
    columnCount = c->countingColumns(file_name);
    s = c->tabulating(m, file_name, columnCount, analysis, arithmetic);

// mean or variance
    n = strcspn( s.c_str(), "0123456789");
    s = s.substr(n);
    n = strspn( s.c_str(), ".0123456789");

    floating_point_number = s.substr(0, n);
    statusBarMessage(floating_point_number);

// table
    n = s.find(COLON_GAP);
    s = s.substr( n - 2 );
    areaTextBuffer->text( s.c_str() );
    delete c;
}

void Rannu::diagonalHandler() {
    const int ADJUSTER = 22;

    checkManageMemory();
    whichCorner = 3;

    if(rannu_w > MAIN_WIDTH) {
        rannu_h = MAIN_HEIGHT;
        rannu_w = MAIN_WIDTH;
        tracking();
    }

    rannu_x = screen_width - rannu_w;
    rannu_y = LARGE_PANE_Y + ADJUSTER - rannu_h;
    resize(rannu_x, rannu_y, rannu_w, rannu_h);

    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->whichCorner = whichCorner;
}

void Rannu::groupingHandler() {
    std::string INTERVAL_0_63("interval of 0-63");

    grouping = true;
    interval_0 = 0;
    interval_1 = 63;

    statusBarMessage(INTERVAL_0_63);
    mm->grouping = grouping;
    mm->interval_0 = interval_0;
    mm->interval_1 = interval_1;
}

void Rannu::helpHandler() {
    versionPane->helpHandler();
}

void Rannu::largeHandler() {
    checkManageMemory();
    whichCorner = 1;

    const int COMBO_HEIGHT = STATUS_BAR_HEIGHT;
    const int COMBO_WIDTH = LARGE_PANE_X/2;
    const int TEXT_AREA_WIDTH = LARGE_PANE_X;
    const int TEXT_AREA_Y = MENU_BAR_HEIGHT + COMBO_HEIGHT;
    const int TEXT_AREA_HEIGHT = LARGE_PANE_Y - STATUS_BAR_HEIGHT - TEXT_AREA_Y;

    rannu_h = LARGE_PANE_Y;
    rannu_w = LARGE_PANE_X;
    rannu_x = CORNER_X;
    rannu_y = CORNER_Y;
    rannu->resize(rannu_x, rannu_y, rannu_w, rannu_h);
    menu->resize(0, 0, LARGE_PANE_X, MENU_BAR_HEIGHT);

    combo->resize(0, MENU_BAR_HEIGHT, COMBO_WIDTH, COMBO_HEIGHT);
    countButton->resize(COMBO_WIDTH, MENU_BAR_HEIGHT, BUTTON_WIDTH, BUTTON_HEIGHT);

    int x = LARGE_PANE_X - BUTTON_WIDTH;
    largeButton->resize(x, MENU_BAR_HEIGHT, BUTTON_WIDTH, BUTTON_HEIGHT);

    textArea->resize(0, TEXT_AREA_Y, LARGE_PANE_X, TEXT_AREA_HEIGHT);
    statusBar->resize(0, LARGE_PANE_Y - STATUS_BAR_HEIGHT, LARGE_PANE_X, STATUS_BAR_HEIGHT);
    rannu->redraw();

    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->rannu_w = rannu_w;
    mm->rannu_h = rannu_h;
    mm->whichCorner = whichCorner;
}

void Rannu::lotoHandler(int a) {
    using namespace std;

    string INTERVAL_1_31("interval of 1-31");
    string INTERVAL_1_37("interval of 1-37");
    string INTERVAL_1_40("interval of 1-40");
    string INTERVAL_1_43("interval of 1-43");

    switch(a) {
    case 31:
        statusBarMessage(INTERVAL_1_31);
        break;
    case 37:
        statusBarMessage(INTERVAL_1_37);
        break;
    case 40:
        statusBarMessage(INTERVAL_1_40);
        break;
    case 43:
        statusBarMessage(INTERVAL_1_43);
        break;
    default:
        a = 37;
        statusBarMessage(INTERVAL_1_37);
        break;
    }

    grouping = false;
    interval_0 = 1;
    interval_1 = a;
    mm->grouping = grouping;
    mm->interval_0 = interval_0;
    mm->interval_1 = interval_1;
}

void Rannu::manageHandler() {
    const bool NO_BUILDING = false;
    managePane->setting(NO_BUILDING);
    managePane->centering();
    managePane->resizable(NULL);
    managePane->set_modal();
    managePane->show();
}

void Rannu::redraw() {
    Fl_Window::redraw();
}

void Rannu::rightHandler() {
    const int ADJUSTER = 22;

    checkManageMemory();
    whichCorner = 2;

    if(rannu_w > MAIN_WIDTH) {
        rannu_h = MAIN_HEIGHT;
        rannu_w = MAIN_WIDTH;
        tracking();
    }

    rannu_x = screen_width - rannu_w;
    rannu_y = CORNER_Y;
    resize(rannu_x, rannu_y, rannu_w, rannu_h);

    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->whichCorner = whichCorner;
}

void Rannu::selectingPRNG(int r) {
    const int MERSENNE_TWISTER = 1;
    const int XORSHIFT = 6;

    switch(r) {
    case XORSHIFT:
        comboTextBuffer->text("xorshift");
        prng = XORSHIFT;
        break;
    default:
        if(rannu_w > SMALL_PANE_X) {
            comboTextBuffer->text("Mersenne Twister (MT)");
        } else {
            comboTextBuffer->text("MT");
        }

        prng = MERSENNE_TWISTER;
        break;
    }

    mm->prng = prng;
}

void Rannu::setStatusBar(std::string s) {
    statusBarText = s;
    statusBarTextBuffer->text( s.c_str() );
}

void Rannu::smallHandler() {
    checkManageMemory();
    whichCorner = 0;

    const int COMBO_HEIGHT = STATUS_BAR_HEIGHT;
    const int COMBO_WIDTH = SMALL_PANE_WIDTH/2;
    const int MERSENNE_TWISTER = 1;
    const int TEXT_AREA_WIDTH = SMALL_PANE_WIDTH;
    const int TEXT_AREA_Y = MENU_BAR_HEIGHT + COMBO_HEIGHT;
    const int TEXT_AREA_HEIGHT = SMALL_PANE_HEIGHT - STATUS_BAR_HEIGHT - TEXT_AREA_Y;

    rannu_h = SMALL_PANE_HEIGHT;
    rannu_w = SMALL_PANE_WIDTH;
    rannu_x = SMALL_PANE_X;
    rannu_y = SMALL_PANE_Y;
    rannu->resize(rannu_x, rannu_y, rannu_w, rannu_h);
    menu->resize(0, 0, SMALL_PANE_WIDTH, MENU_BAR_HEIGHT);

    if(prng == MERSENNE_TWISTER) {
        comboTextBuffer->text("MT");
    }

    combo->resize(0, MENU_BAR_HEIGHT, COMBO_WIDTH, COMBO_HEIGHT);
    countButton->resize(COMBO_WIDTH, MENU_BAR_HEIGHT, BUTTON_WIDTH, BUTTON_HEIGHT);

    int x = SMALL_PANE_WIDTH - BUTTON_WIDTH;
    largeButton->resize(x, MENU_BAR_HEIGHT, BUTTON_WIDTH, BUTTON_HEIGHT);

    textArea->resize(0, TEXT_AREA_Y, SMALL_PANE_WIDTH, TEXT_AREA_HEIGHT);
    statusBar->resize(0, SMALL_PANE_HEIGHT - STATUS_BAR_HEIGHT, SMALL_PANE_WIDTH, STATUS_BAR_HEIGHT);
    rannu->redraw();

    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->rannu_w = rannu_w;
    mm->rannu_h = rannu_h;
    mm->whichCorner = whichCorner;
}

void Rannu::standardHandler() {
    checkManageMemory();
    whichCorner = 0;
    rannu_h = MAIN_HEIGHT;
    rannu_w = MAIN_WIDTH;
    resize(rannu_x, rannu_y, rannu_w, rannu_h);

    screen_width = centering();
    standard_main_x = x();
    standard_main_y = y();
    rannu_x = standard_main_x;
    rannu_y = standard_main_y;

    tracking();

    mm->rannu_h = rannu_h;
    mm->rannu_w = rannu_w;
    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->whichCorner = whichCorner;
}

void Rannu::statusBarMessage(std::string & s) {
    statusBarText = s;
    statusBarTextBuffer->text( s.c_str() );
}

void Rannu::stringArrayToValues() {
    char *stop;

    csv = (ZERO == m[1]) ? false : true;
    pathName = m[2] + RANNU_TEXT;
    kilo = strtol( m[3].c_str(), &stop, 10);
    interval_0 = stringToInteger(m[4]);
    interval_1 = stringToInteger(m[5]);
    grouping = (ZERO == m[6]) ? false : true;
    from = stringToInteger(m[7]);
    seed = strtol( m[8].c_str(), &stop, 10);
    lineCount = stringToInteger(m[9]);

    mm->csv = csv;
    mm->from = from;
    mm->grouping = grouping;
    mm->interval_0 = interval_0;
    mm->interval_1 = interval_1;
    mm->kilo = kilo;
    mm->lineCount = lineCount;
    mm->pathName = pathName;
    mm->pleaseCheck = true;
    mm->seed = seed;
}

void Rannu::temporaryFileName(char *name) {
    statusBarTextBuffer->text(name);
}

void Rannu::timerHandler() {
    const bool NO_BUILDING = false;
    checkManageMemory();

    if(showWhere) {
// Showing where the cursor is
        where();
    }

// Checking if MANAGE_TEXT has been modified
    long m = modified_file(MANAGE_TEXT);
    if(m != untouched) {
        untouched = m;
        readManageText();
        statusBarMessage(FULL_PATH_NAME);
        managePane->setting(NO_BUILDING);
    }

    Fl::repeat_timeout(NONSAMPLING_PERIOD, timer_cb);
}

void Rannu::tracking() {
    checkManageMemory();

    const int COMBO_HEIGHT = STATUS_BAR_HEIGHT;
    const int COMBO_WIDTH = rannu_w/2;
    const int TEXT_AREA_WIDTH = rannu_w;
    const int TEXT_AREA_Y = MENU_BAR_HEIGHT + COMBO_HEIGHT;
    const int TEXT_AREA_HEIGHT = rannu_h - STATUS_BAR_HEIGHT - TEXT_AREA_Y;

    menu->resize(0, 0, rannu_w, MENU_BAR_HEIGHT);

    combo->resize(0, MENU_BAR_HEIGHT, COMBO_WIDTH, COMBO_HEIGHT);
    countButton->resize(COMBO_WIDTH, MENU_BAR_HEIGHT, BUTTON_WIDTH, BUTTON_HEIGHT);

    int x = rannu_w - BUTTON_WIDTH;
    largeButton->resize(x, MENU_BAR_HEIGHT, BUTTON_WIDTH, BUTTON_HEIGHT);

    textArea->resize(0, TEXT_AREA_Y, rannu_w, TEXT_AREA_HEIGHT);
    statusBar->resize(0, rannu_h - STATUS_BAR_HEIGHT, rannu_w, STATUS_BAR_HEIGHT);
    rannu->redraw();

    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->rannu_w = rannu_w;
    mm->rannu_h = rannu_h;
}

void Rannu::underHandler() {
    const int ADJUSTER = 22;

    checkManageMemory();
    whichCorner = 4;
    rannu_x = CORNER_X;
    rannu_y = LARGE_PANE_Y + ADJUSTER - rannu_h;
    resize(rannu_x, rannu_y, rannu_w, rannu_h);

    mm->rannu_x = rannu_x;
    mm->rannu_y = rannu_y;
    mm->whichCorner = whichCorner;
}

void Rannu::versionHandler() {
    using namespace std;

    stringstream os;
    os << "Rannu " << VERSION << " using FLTK 1.3.0";
    string s = os.str();
    statusBarTextBuffer->text( s.c_str() );
    versionPane->centering();
    versionPane->resizable(NULL);
    versionPane->set_modal();
    versionPane->show();
}

void redraw_cb(Fl_Widget *, void *) {
    rannu->redraw();
}

void right_cb(Fl_Widget *, void *) {
    rannu->rightHandler();
}

void small_cb(Fl_Widget *, void *) {
    rannu->smallHandler();
}

void standard_cb(Fl_Widget *, void *) {
    rannu->standardHandler();
}

void text_cb(Fl_Widget *, void *) {
    mm->csv = false;
    mm->pleaseCheck = true;
    rannu->textHandler();
}

void timer_cb(void *) {
    rannu->timerHandler();
}

void under_cb(Fl_Widget *, void *) {
    rannu->underHandler();
}

void version_cb(Fl_Widget *, void *) {
    rannu->versionHandler();
}

void xorshift_cb(Fl_Widget *, void *) {
    const int XORSHIFT = 6;
    rannu->selectingPRNG(XORSHIFT);
}
