#ifndef RANNU_MAIN_H
#define RANNU_MAIN_H

#include <FL/Fl.H>
#include <FL/Fl_Bitmap.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Timer.H>
#include <FL/Fl_Widget.H>
#include <FL/Fl_Window.H>
#include <FL/x.H>
#include <fstream>
#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <string>
#include <time.h>
#include "rannu.h"

typedef Fl_Button *FL_BUTTON;
typedef Fl_Input *FL_INPUT;
typedef Fl_Text_Buffer *FL_TEXT_BUFFER;
typedef Fl_Text_Display *FL_TEXT_DISPLAY;

class VersionPane : public Fl_Double_Window {
    FL_BUTTON bitmapButton, closeButton, copyrightButton, helpButton, rannuFltkButton;

public:
    ~VersionPane(){}
    VersionPane(int W, int H, const char *t) : Fl_Double_Window(W, H, t) {} 

    bool setting();
    void centering();
    void helpHandler();
};

class ManagePane : public Fl_Double_Window {
    bool csv, grouping, produce_rannu_text;
    char buffer[BUFFER_SIZE];
    const std::string CLOSE_BRACKET, NEW_LINE, ONE, RANNU_CSV, RANNU_TEXT, ZERO;
    int from, interval_0, interval_1, lineCount, prng;
    int field[8];
    long kilo, seed, untouched;
    FL_BUTTON captionButton, closeButton, helpButton, kilobytesButton, linesButton, prngButton, rannuCsvButton, rannuTextButton, saveButton, whereButton;
    FL_INPUT input2, input3, input4, input5, input6, input7;
    Fl_Text_Buffer *text_buffer;
    Fl_Text_Display *display;
    std::string error, FULL_PATH_NAME, path, pathName;
    std::string m[ MANAGE_TEXT_LINE_COUNT + 1 ];

    bool readManageText();
    bool where();
    bool writeManageText();
    char* right_justifying(char *p);
    char* right_justifying(int value);
    char* right_justifying(long value);

public:
    ~ManagePane(){}
    ManagePane(int W, int H, const char *t);
    int centering();
    void fileOpenDialog();
    void focusHandler();
    void KBhandler();
    void lineCountHandler();
    void rannuCsvHandler();
    void rannuTextHandler();
    void saveHandler();
    void setting(bool building);
};

class Rannu : public Fl_Window {
    bool csv, grouping, produce_rannu_text;
    const int STATUS_BAR_TEXT_LIMIT, TEXT_LIMIT;
    const std::string COMMA, EMPTY, NEW_LINE;
    Fl_Boxtype diamond_square;
    FL_BUTTON countButton, largeButton;
    Fl_Menu_Bar *menu;
    FL_TEXT_BUFFER areaTextBuffer, comboTextBuffer, statusBarTextBuffer;
    FL_TEXT_DISPLAY combo, statusBar, textArea;
    int from, interval_0, interval_1, lineCount, prng, rannu_h, rannu_w, rannu_x, rannu_y, screen_width, standard_main_x, standard_main_y, whichCorner;
    long kilo, seed;
    std::string pathName, statusBarText;
    std::string m[ MANAGE_TEXT_LINE_COUNT + 1 ];
    VersionPane *versionPane;

    bool header(std::string & s);
    bool readManageText();
    bool where();
    void checkManageMemory();
    void setStatusBar(std::string s);
    void stringArrayToValues();

public:
    ~Rannu(){}
    Rannu(int W, int H, const char *t);

    bool setting(VersionPane *v);
    bool textHandler();
    int centering();
    void centerHandler();
    void copyHandler();
    void cornerHandler();
    void cou99(bool arithmetic);
    void countHandler();
    void diagonalHandler();
    void groupingHandler();
    void helpHandler();
    void largeHandler();
    void lotoHandler(int a);
    void manageHandler();
    void redraw();
    void rightHandler();
    void selectingPRNG(int r);
    void setStatusBar();
    void smallHandler();
    void standardHandler();
    void statusBarMessage(std::string & s);
    void temporaryFileName(char *name);
    void timerHandler();
    void tracking();
    void underHandler();
    void versionHandler();
};

#endif // RANNU_MAIN_H
